/**
  ******************************************************************************
  * @file    MPU6050.c
  * @author  Waveshare Team
  * @version V1.0
  * @date    29-August-2014
  * @brief   This file includes the MPU6050 driver functions
  
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, WAVESHARE SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2014 Waveshare</center></h2>
  ******************************************************************************
  */


#include "MPU6050.h"
#include "I2C.h"


MPU6050_TypeDef MPU6050_Offset;

/**
  * @brief  Initializes MPU6050 GPIO
  * @param  None
  * @retval None
  */

void MPU6050_GPIOInit(void)
{
	GPIO_InitTypeDef  GPIO_InitStructure;
	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_6;				 		 
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU; 
	GPIO_Init(GPIOA, &GPIO_InitStructure);	
}


/**
  * @brief  Check MPU6050,ensure communication succeed
  * @param  None
  * @retval true: communicate succeed
  *               false: communicate fualt 
  */
bool MPU6050_Check(void) 
{
   	if(MPU6050_ADDRESS_AD0_LOW == I2C_ReadOneByte(MPU6050_DEVICE_ADDR, MPU6050_RA_WHO_AM_I))  
   	{
   		return true;
   	}
   	else 
   	{
   		return false;
   	}
}

/**
  * @brief  Digital filter
  * @param *pIndex:
  * @param *pAvgBuffer:
  * @param InVal:
  * @param pOutVal:
  *
  * @retval None
  *               
  */
void MPU6050_CalAvgValue(uint8_t *pIndex, int16_t *pAvgBuffer, int16_t InVal, int32_t *pOutVal)
{	
	uint8_t i;
	
  	*(pAvgBuffer + ((*pIndex) ++)) = InVal;
  	*pIndex &= 0x07;
  	
  	*pOutVal = 0;
	for(i = 0; i < 8; i ++) 
  	{
    	*pOutVal += *(pAvgBuffer + i);
  	}
  	*pOutVal >>= 3;
}

/**
  * @brief  configure the clock soure
  * @param  source: see Register 107 C Power Management 1 CLKSEL[2:0]
  * @retval None
  */
void MPU6050_SetClockSource(uint8_t source)
{
    I2C_WriteBits(MPU6050_DEVICE_ADDR, MPU6050_RA_PWR_MGMT_1, MPU6050_PWR1_CLKSEL_BIT, MPU6050_PWR1_CLKSEL_LENGTH, source);
}

/**
   * @brief  Configure the gyroscopes' full range.
   * @param  range: see Register 27 C Gyroscope Configuration FS_SEL[1:0] 
   * @retval None
   */
void MPU6050_SetFullScaleGyroRange(uint8_t range)
{
    I2C_WriteBits(MPU6050_DEVICE_ADDR, MPU6050_RA_GYRO_CONFIG, MPU6050_GCONFIG_FS_SEL_BIT, MPU6050_GCONFIG_FS_SEL_LENGTH, range);
}

/**
  * @brief  Configure  the  accelerometer  full  scale range
  * @param  range: see Register 28 C Accelerometer Configuration AFS_SEL[1:0] 
  * @retval None
  */
void MPU6050_SetFullScaleAccelRange(uint8_t range)
{
    I2C_WriteBits(MPU6050_DEVICE_ADDR, MPU6050_RA_ACCEL_CONFIG, MPU6050_ACONFIG_AFS_SEL_BIT, MPU6050_ACONFIG_AFS_SEL_LENGTH, range);
}

/**
  * @brief  Configure the power mode
  * @param  enabled: see Register 107 C Power Management 1 SLEEP
  * @retval None
  */
void MPU6050_SetSleepEnabled(uint8_t enabled) 
{
    I2C_WriteOneBit(MPU6050_DEVICE_ADDR, MPU6050_RA_PWR_MGMT_1, MPU6050_PWR1_SLEEP_BIT, enabled);
}

/**
  * @brief  Enable or disable I2C Master  Mode
  * @param  enabled: see Register 106 C User Control  I2C_MST_EN
  * @retval None
  */

void MPU6050_SetI2CMasterModeEnabled(uint8_t enabled) 
{
    I2C_WriteOneBit(MPU6050_DEVICE_ADDR, MPU6050_RA_USER_CTRL, MPU6050_USERCTRL_I2C_MST_EN_BIT, enabled);
}

/**
  * @brief  Enable or disable Bypass Mode on the I2C Master
  * @param  enabled: see Register 55 C INT Pin / Bypass Enable Configuration   I2C_BYPASS_EN
  * @retval None
  */

void MPU6050_SetI2CBypassEnabled(uint8_t enabled) 
{
    I2C_WriteOneBit(MPU6050_DEVICE_ADDR, MPU6050_RA_INT_PIN_CFG, MPU6050_INTCFG_I2C_BYPASS_EN_BIT, enabled);
}

/**
  * @brief  Data outputs Standby
  * @param  None
  * @retval true:Standby
  *               false: fualt
  */

bool MPU6050_Standby(void)
{
    if(Bit_SET == GPIO_ReadInputDataBit(GPIOA, GPIO_Pin_6))
    {
	  	return true;
	}
	else 
	{
		return false;
	}
}

/**
  * @brief Get motion datas
  * @param  *ax: accelerometer X
  * @param  *ay: accelerometer Y
  * @param  *az: accelerometer Z
  * @param  *gx: gyroscopes X
  * @param  *gy: gyroscopes Y
  * @param  *gz: gyroscopes Z
  * @retval None
  */
void MPU6050_GetMotion(int16_t* ax, int16_t* ay, int16_t* az, int16_t* gx, int16_t* gy, int16_t* gz) 
{
	uint8_t i, Buffer[14] = {0};
	int16_t InBuffer[6] = {0}; 
	static int32_t OutBuffer[6] = {0};
	static MPU6050_AvgTypeDef MPU6050_Filter[6];
	
	if(MPU6050_Standby())
	{
		I2C_ReadBuff(MPU6050_DEVICE_ADDR, MPU6050_RA_ACCEL_XOUT_H, 14, Buffer);
		
	    InBuffer[0] = (((int16_t)Buffer[0]) << 8) | Buffer[1];
	    InBuffer[1] = (((int16_t)Buffer[2]) << 8) | Buffer[3];
	    InBuffer[2] = (((int16_t)Buffer[4]) << 8) | Buffer[5];

	    InBuffer[3] = (((int16_t)Buffer[8]) << 8) | Buffer[9];
	    InBuffer[4] = (((int16_t)Buffer[10]) << 8) | Buffer[11];
	    InBuffer[5] = (((int16_t)Buffer[12]) << 8) | Buffer[13];

		for(i = 0; i < 6; i ++)	
		{
			MPU6050_CalAvgValue(&MPU6050_Filter[i].Index, MPU6050_Filter[i].AvgBuffer, InBuffer[i], OutBuffer + i);
		}
	}

	*ax = *(OutBuffer + 0);
	*ay = *(OutBuffer + 1);
	*az = *(OutBuffer + 2);
	*gx = *(OutBuffer + 3) - MPU6050_Offset.X;
	*gy = *(OutBuffer + 4) - MPU6050_Offset.Y;
	*gz = *(OutBuffer + 5) - MPU6050_Offset.Z;
}


/**
  * @brief  Initializes gyroscopes offset
  * @param  None
  * @retval None
  */

void MPU6050_InitGyrOffset(void)
{
	uint8_t i;
	int16_t Temp[6] = {0};
	int32_t	TempGx = 0, TempGy = 0, TempGz = 0;
	
 	for(i = 0; i < 32; i ++)
 	{
		MPU6050_GetMotion(&Temp[0], &Temp[1], &Temp[2], 
						  &Temp[3], &Temp[4], &Temp[5]);
		
		TempGx += Temp[3];
		TempGy += Temp[4];
		TempGz += Temp[5];

		Delay_us(100);
	}

	MPU6050_Offset.X = TempGx >> 5;
	MPU6050_Offset.Y = TempGy >> 5;
	MPU6050_Offset.Z = TempGz >> 5;

}

/**
  * @brief  Initializes MPU6050
  * @param  None
  * @retval None
  */
void MPU6050_Init(void)
{
	MPU6050_GPIOInit();

    MPU6050_SetClockSource(MPU6050_CLOCK_PLL_XGYRO); 
    MPU6050_SetFullScaleGyroRange(MPU6050_GYRO_FS_1000);
    MPU6050_SetFullScaleAccelRange(MPU6050_ACCEL_FS_2);	
    MPU6050_SetSleepEnabled(0); 
	MPU6050_SetI2CMasterModeEnabled(0);	 
	MPU6050_SetI2CBypassEnabled(1);	 
	
	I2C_WriteOneBit(MPU6050_DEVICE_ADDR, MPU6050_RA_INT_PIN_CFG, MPU6050_INTCFG_INT_LEVEL_BIT, 0);
	I2C_WriteOneBit(MPU6050_DEVICE_ADDR, MPU6050_RA_INT_PIN_CFG, MPU6050_INTCFG_INT_OPEN_BIT, 0);
	I2C_WriteOneBit(MPU6050_DEVICE_ADDR, MPU6050_RA_INT_PIN_CFG, MPU6050_INTCFG_LATCH_INT_EN_BIT, 1);
	I2C_WriteOneBit(MPU6050_DEVICE_ADDR, MPU6050_RA_INT_PIN_CFG, MPU6050_INTCFG_INT_RD_CLEAR_BIT, 1);
    I2C_WriteOneBit(MPU6050_DEVICE_ADDR, MPU6050_RA_INT_ENABLE, MPU6050_INTERRUPT_DATA_RDY_BIT, 1);

	if(MPU6050_Check())
	{
		GPIO_SetBits(GPIOB, GPIO_Pin_13);
	}
	else
	{
		GPIO_ResetBits(GPIOB, GPIO_Pin_13);
	}

	MPU6050_InitGyrOffset();
}

/******************* (C) COPYRIGHT 2014 Waveshare *****END OF FILE*******************/

